#!/bin/bash
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of datasid
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# set PREFIX
export PREFIX="$(readlink -f "$(dirname $0)/../")"

# run datasid-common.sh
source "$(readlink -f "$(dirname $0)")/datasid-common.sh" || exit 1

if test -z "${PREFIX}"; then
    date +"%F %T - ERROR: Prefix not set."
    exit 2
fi

PROXYPASSWD="--proxy-password"
if which wget &> /dev/null; then
    WGETBIN="$(which wget)"
    WGETVERSION="$(${WGETBIN} --version | head -n1 | cut -d' ' -f3)"
    WGETVER1="$(echo ${WGETVERSION} | cut -d'.' -f1)"
    WGETVER2="$(echo ${WGETVERSION} | cut -d'.' -f2)"
    # If the system wget(1) is older than the 1.10 version, use
    # the option "--proxy-passwd" to pass the proxy password to wget(1)
    if test ${WGETVER1} -lt 1 -o ${WGETVER1} -eq 1 -a ${WGETVER2} -lt 10
    then
        PROXYPASSWD="--proxy-passwd"
    fi
else
    WGETBIN="${PREFIX}/bin/datasid-builtin-wget"
fi

if test -n "${PROXY}"; then
    http_proxy="${PROXY}" ${WGETBIN} --trie=${UPDATETRIES} \
                                     --timeout=${UPDATETIMEOUT} \
                                     --proxy-user="${PROXY_USERNAME}" \
                                     ${PROXYPASSWD}="${PROXY_PASSWORD}" \
                                     $* || exit 1
else
    ${WGETBIN} --tries=${UPDATETRIES} \
               --timeout=${UPDATETIMEOUT} \
               $* || exit 1
fi
