﻿# Copyright (C) 2009-2012 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of windows-coleta
#
# windows-coleta is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

#coding: utf-8
import os
import _winreg
import ctypes
import wmi
import socket
import struct
from xml.etree.ElementTree import Element, SubElement, Comment, tostring

DATASIDPATH = os.environ["ProgramFiles"] + "\\DataSID"
# Return the value from key/subkey in windows registry
def getRegistryValue(key, subkey, value):
    key = getattr(_winreg, key)
    handle = _winreg.OpenKey(key, subkey)
    (value, type) = _winreg.QueryValueEx(handle, value)
    return value

def getTelecentroInfo(info):
    tlInfoFile = open(DATASIDPATH + "\\conf\\telecentroInfo", "r")
    tlInfo = value = " "
    while (tlInfo != info and tlInfo != ""):
	try:
		(tlInfo, value) = tlInfoFile.readline().split("=")
	except:
		tlInfo = value = ""
    tlInfoFile.close()
    return value

# Return Windows version
def getOSInfo(key):
    return getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\Microsoft\\Windows NT\\CurrentVersion", key)

# Return CPU model installed in the machine
def getCpu():
    return getRegistryValue("HKEY_LOCAL_MACHINE", "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0", "ProcessorNameString")

# Return the machine mac address
def getMac(host):
    # Check for api availability
    try:
        SendARP = ctypes.windll.Iphlpapi.SendARP
    except:
        raise NotImplementedError('Usage only on Windows 2000 and above')

    # Doesn't work with loopbacks
    if host == '127.0.0.1' or host.lower() == 'localhost':
        host = socket.gethostname()

    try:
        inetaddr = ctypes.windll.wsock32.inet_addr(host)
        if inetaddr in (0, -1):
            raise Exception
    except:
        hostip = socket.gethostbyname(host)
        inetaddr = ctypes.windll.wsock32.inet_addr(hostip)

    buffer = ctypes.c_buffer(6)
    addlen = ctypes.c_ulong(ctypes.sizeof(buffer))

    if SendARP (inetaddr, 0, ctypes.byref(buffer),
                    ctypes.byref(addlen)) != 0:
        raise WindowsError('Retrieval of mac address(%s) - failed' % host)

    # Convert binary data into a string
    macaddr = ''

    for intval in struct.unpack('BBBBBB', buffer):
        if intval > 15:
            replacestr = '0x'
        else:
            replacestr = 'x'
        macaddr = ''.join([macaddr, hex(intval).replace(replacestr, '' ), ":"],)
    return macaddr.upper()

# Return the total physical memory installed in the machine
def getRam():
    c = wmi.WMI()
    mem = 0
    for i in c.Win32_PhysicalMemory():
        mem = mem + int (i.Capacity)
    return mem / 1024

# Return disk model
def getHDDModel(disk):
    return disk.Model

# Return disk size in Gigabytes
def getHDDSize(disk):
    return int(disk.Size) / (1024 * 1024 * 1024)

# Return the total size of non Windows partitions
def nonWindowsPartitionSize(c):
    size = 0
    for partition in c.Win32_DiskPartition():
        if partition.Description != "Installable File System":
            if partition.Description != "Unknown":
                size = size + int (partition.Size)
    return size

# Return the total used disk
def diskUsed(c):
    free = 0
    size = 0
    for disk in c.Win32_LogicalDisk (DriveType = 3):
        free = free + int (disk.FreeSpace)
        size = size + int (disk.Size)
    return size - free

# Main collect function. This function gets the machine information and creates
# the XML
def collect():
    mainTag = Element('collected-data')

    # Collect telecentros's info
    admin_name = getTelecentroInfo("responsavel")
    tl_connection = getTelecentroInfo("conexao")
    tl_name = getTelecentroInfo("nome")
    state = getTelecentroInfo("estado")
    city = getTelecentroInfo("municipio")
    tl_street = getTelecentroInfo("rua")
    tl_zipcode = getTelecentroInfo("cep")
    tl_beneficiary = getTelecentroInfo("beneficiaria")
    tl_number = getTelecentroInfo("numero")
    tl_neighborhood = getTelecentroInfo("bairro")
    geolocation = getTelecentroInfo("geolocalizacao")
    superid = getTelecentroInfo("superid")
    user_count = getTelecentroInfo("usuarios")

    tagTelecentroInfo = SubElement(mainTag, 'telecentro-info')
    tagAdminName = SubElement(tagTelecentroInfo, 'admin_name')
    tagAdminName.text = admin_name
    tagTlConnection = SubElement(tagTelecentroInfo, 'tl_connection')
    tagTlConnection.text = tl_connection
    tagTlName = SubElement(tagTelecentroInfo, 'tl_name')
    tagTlName.text = tl_name
    tagState = SubElement(tagTelecentroInfo, 'state')
    tagState.text = state
    tagCity = SubElement(tagTelecentroInfo, 'city')
    tagCity.text = city
    tagTlStreet = SubElement(tagTelecentroInfo, 'tl_street')
    tagTlStreet.text = tl_street
    tagTlNumber = SubElement(tagTelecentroInfo, 'tl_number')
    tagTlNumber.text = tl_number
    tagTlZipcode = SubElement(tagTelecentroInfo, 'tl_zipcode')
    tagTlZipcode.text = tl_zipcode
    tagTlBeneficiary = SubElement(tagTelecentroInfo, 'tl_beneficiary')
    tagTlBeneficiary.text = tl_beneficiary
    tagTlNeighborhood = SubElement(tagTelecentroInfo, 'tl_neighborhood')
    tagTlNeighborhood.text = tl_neighborhood
    tagGeolocation = SubElement(tagTelecentroInfo, 'geolocation')
    tagGeolocation.text = geolocation
    tagSuperid = SubElement(tagTelecentroInfo, 'superid')
    tagSuperid.text = superid
    tagtUserCount = SubElement(tagTelecentroInfo, 'user_count')
    tagtUserCount.text = user_count

    # Collect Version
    tagAgent = SubElement(mainTag, "agent-version")
    tagAgent.text = getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\DataSID", "Version")    

    # Inventory
    tagInventory = SubElement(mainTag, 'inventory')

    # Collect Hard Disk
    c = wmi.WMI()
    tagInventoryDisks = SubElement (tagInventory, 'disks', {'type':'tree'})
    
    i = -1
    for disk in c.win32_DiskDrive():
        if disk.MediaType != "Removable Media":
            i = i + 1
            hddSize = getHDDSize(disk)
            hddModel = getHDDModel(disk)
            used = nonWindowsPartitionSize(c) + diskUsed(c)
            used = used / (1024 * 1024 * 1024)
            tagInventoryDisk = SubElement(tagInventoryDisks, 'disk')
            tagInventoryId = SubElement(tagInventoryDisk, 'id')
            tagInventoryId.text = str(i)
            tagInventoryDiskValue1 = SubElement(tagInventoryDisk, 'model')
            tagInventoryDiskValue1.text = hddModel
            tagInventoryDiskValue2 = SubElement(tagInventoryDisk, 'size')
            tagInventoryDiskValue2.text = str(hddSize)
            tagInventoryDiskValue3 = SubElement(tagInventoryDisk, 'used')
            tagInventoryDiskValue3.text = str(used)
           
    # Collect Memory
    memory = getRam()

    tagInventoryMemory = SubElement(tagInventory, 'memory')
    tagInventoryMemory.text = str(memory)     

    # Collect OS
    os = getOSInfo("ProductName")

    tagInventoryDistro = SubElement(tagInventory, 'distro')
    tagInventoryDistro.text = os
    tagInventoryKernel = SubElement(tagInventory, 'kernel')
    tagInventoryKernel.text = 'Windows'
    tagInventoryOs = SubElement(tagInventory, 'os')
    tagInventoryOs.text = 'Windows'

    # Collect CPU
    cpuVersion = getCpu()

    tagInventoryCPU = SubElement ( tagInventory, 'processor')
    tagInventoryCPU.text = cpuVersion

    # Collect Mac Address
    macAddress = getMac('localhost')
    macAddress = macAddress[0:-1]

    tagMac = SubElement(mainTag, 'interfaces')
    tagMacValue = SubElement(tagMac, 'interface', {'name':'', 'mac-addres':macAddress})

    tagMachineType = SubElement(mainTag, 'machine-type')
    tagMachineType.text = 'client'

    tagMirrorsTimestamp = SubElement(mainTag, 'mirrors-timestamp')
    tagMirrorsTimestamp.text = " "

    tagUserHistory = SubElement(mainTag, 'user-history')
    tagUserHistory.text = " "

    return mainTag

# Main collect function. This function gets the net usage information and creates
# the XML    
def netCollect():
    mainTag = Element('collect')

    # Collect INEP
    inep = getINEP()

    tagInep = SubElement(mainTag, 'inep')
    tagInepValue = SubElement(tagInep, 'number', {'type':'int', 'value':inep})	
    
    # Collect Version
    tagAgent = SubElement(mainTag, 'agent')
    tagVersion = SubElement(tagAgent, 'version', {'type':'string', 'value':getRegistryValue("HKEY_LOCAL_MACHINE", "SOFTWARE\\DataSID", "Version")})
	
    # Collect Mac Address
    macAddress = getMac('localhost')
    macAddress = macAddress[0:-1]
    
    tagMac = SubElement(mainTag, 'mac-address')
    tagMacValue = SubElement(tagMac, 'mac', {'type':'string', 'value':macAddress})
    
    tagProject = SubElement( mainTag, 'project')
    tagProjectSource = SubElement(tagProject, 'source', {'type':'int', 'value':"0"})
    
    tagUse = SubElement(mainTag, 'use')
    tagUseBandUsage = SubElement(tagUse, 'bandwidth-usage', {'type':'tree'})
    
    for i in range(1, 289):
        tagNetUse = SubElement(tagUseBandUsage, 'netuse', {'id':str(i)})
        tagDate = SubElement(tagNetUse, 'date', {'value':'0001-01-01', 'type':'string'})
        tagTime = SubElement(tagNetUse, 'time', {'value':'0:0', 'type':'string'})
        
        tagRx = SubElement(tagNetUse, 'rx')
        tagPackets = SubElement(tagRx, 'packets', {'value':"0", 'type':'int'})
        tagBytes = SubElement(tagRx, 'bytes', {'value':"0", 'type':'int'})
        
        tagTx = SubElement(tagNetUse, 'tx')
        tagPackets = SubElement(tagTx, 'packets', {'value':"0", 'type':'int'})
        tagBytes = SubElement(tagTx, 'bytes', {'value':"0", 'type':'int'})
        	
    return mainTag
